@@ -87,11 +87,15 @@ object DataFrameOps {
 
     def broadcastJoinSkewed(skewed: DataFrame,
                             joinCol: String,
-                            numberCustsToBroadcast: Int): DataFrame =
+                            numberCustsToBroadcast: Int,
+                            filterCnt: Option[Long] = None,
+                            joinType: String = "inner"): DataFrame =
       SparkDFUtils.broadcastJoinSkewed(df,
                                        skewed,
                                        joinCol,
-                                       numberCustsToBroadcast)
+                                       numberCustsToBroadcast,
+                                       filterCnt,
+                                       joinType)
 
     def joinSkewed(notSkewed: DataFrame,
                    joinExprs: Column,
@@ -101,6 +105,9 @@ object DataFrameOps {
   
     def explodeArray(arrayCol: Column,
                      alias: String) =
-      SparkDFUtils.explodeArray(df, arrayCol, alias)  
+      SparkDFUtils.explodeArray(df, arrayCol, alias)
+
+    def dedupRandomN(df: DataFrame, groupCol: Column, maxSize: Int): DataFrame =
+      SparkDFUtils.dedupRandomN(df, groupCol, maxSize)
   }
 }
