@@ -88,11 +88,15 @@ class SparkDFUtilsBridge {
   def broadcastJoinSkewed(notSkewed: DataFrame,
                           skewed: DataFrame,
                           joinCol: String,
-                          numRowsToBroadcast: Int): DataFrame = {
+                          numRowsToBroadcast: Int,
+                          filterCnt: Long,
+                          joinType: String): DataFrame = {
     SparkDFUtils.broadcastJoinSkewed(notSkewed = notSkewed,
-                                     skewed = skewed,
-                                     joinCol = joinCol,
-                                     numRowsToBroadcast = numRowsToBroadcast)
+      skewed = skewed,
+      joinCol = joinCol,
+      numRowsToBroadcast = numRowsToBroadcast,
+      filterCnt = Option(filterCnt),
+      joinType)
   }
 
   def joinWithRange(dfSingle: DataFrame,
@@ -127,6 +131,14 @@ class SparkDFUtilsBridge {
     )
   }
 
+  def explodeArray(df: DataFrame, arrayCol: Column, alias: String): DataFrame = {
+    SparkDFUtils.explodeArray(df, arrayCol, alias)
+  }
+
+  def dedupRandomN(df: DataFrame, groupCol: Column, maxSize: Int): DataFrame = {
+    SparkDFUtils.dedupRandomN(df, groupCol, maxSize)
+  }
+
   private def convertJavaListToSeq[T](list: JavaList[T]): Seq[T] = {
     scala.collection.JavaConverters
       .asScalaIteratorConverter(list.iterator())
@@ -172,7 +184,7 @@ object SparkDFUtils {
   /**
     * Used to get the 'latest' record (after ordering according to the provided order columns)
     * in each group.
-    * the same functionality as {@link #dedup} but implemented using UDAF to utilize
+    * the same functionality as {@link #dedupWithOrder} but implemented using UDAF to utilize
     * map side aggregation.
     * this function should be used in cases when you expect a large number of rows to get combined,
     * as they share the same group column.
@@ -318,17 +330,24 @@ object SparkDFUtils {
     * @param skewed skewed DataFrame
     * @param joinCol join column
     * @param numRowsToBroadcast num of rows to broadcast
+    * @param filterCnt filter out unskewed rows from the boardcast to ease limit calculation
     * @return DataFrame representing the data after the operation
     */
   def broadcastJoinSkewed(notSkewed: DataFrame,
                           skewed: DataFrame,
                           joinCol: String,
-                          numRowsToBroadcast: Int): DataFrame = {
+                          numRowsToBroadcast: Int,
+                          filterCnt: Option[Long] = None,
+                          joinType: String = "inner"): DataFrame = {
     val ss = notSkewed.sparkSession
     import ss.implicits._
-    val skewedKeys = skewed
+    val keyCount = skewed
       .groupBy(joinCol)
       .count()
+    val filteredKeyCount = filterCnt.map(cnt => keyCount.filter($"count" >= cnt)).getOrElse(keyCount)
+
+
+    val skewedKeys = filteredKeyCount
       .sort($"count".desc)
       .limit(numRowsToBroadcast)
       .drop("count")
@@ -339,12 +358,11 @@ object SparkDFUtils {
       .join(broadcast(skewedKeys), $"skew_join_key" === col(joinCol), "left")
       .withColumn("is_skewed_record", col("skew_join_key").isNotNull)
       .drop("skew_join_key")
-      .persist(StorageLevel.DISK_ONLY)
 
     // broadcast map-join, sending the notSkewed data
     val bigRecordsJnd =
       broadcast(notSkewedWithSkewIndicator.filter("is_skewed_record"))
-        .join(skewed, joinCol)
+        .join(skewed.join(broadcast(skewedKeys), $"skew_join_key" === col(joinCol)).drop("skew_join_key"), List(joinCol), joinType)
 
     // regular join for the rest
     val skewedWithoutSkewedKeys = skewed
@@ -353,7 +371,7 @@ object SparkDFUtils {
       .drop("skew_join_key")
     val smallRecordsJnd = notSkewedWithSkewIndicator
       .filter("not is_skewed_record")
-      .join(skewedWithoutSkewedKeys, joinCol)
+      .join(skewedWithoutSkewedKeys, List(joinCol), joinType)
 
     smallRecordsJnd
       .union(bigRecordsJnd)
@@ -542,4 +560,18 @@ object SparkDFUtils {
     val exprs = (0 until arrSize).map(i => arrayCol.getItem(i).alias(s"$alias$i"))
     df.select((col("*") +: exprs):_*)
   }
+
+  /**
+    * Used get the random n records in each group. Uses an efficient implementation
+    * that doesn't order the data so it can handle large amounts of data.
+    *
+    * @param df        DataFrame to operate on
+    * @param groupCol  column to group by the records
+    * @param maxSize   The maximal number of rows per group
+    * @return DataFrame representing the data after the operation
+    */
+  def dedupRandomN(df: DataFrame, groupCol: Column, maxSize: Int): DataFrame = {
+    df.groupBy(groupCol).agg(SparkOverwriteUDAFs.collectLimitedList(expr("struct(*)"), maxSize).as("list"))
+      .select(groupCol,expr("explode(list)"))
+  }
 }
